"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.serialise = exports.parse = exports.isLeafNode = exports.isNotNode = exports.isOrNode = exports.isAndNode = exports.OR = exports.NOT = exports.AND = exports.Label = exports.Subject = exports.From = exports.HasAttachment = void 0;
var nearley_1 = require("nearley");
var grammar = require('./grammar');
function HasAttachment() {
    return { value: 'has:attachment' };
}
exports.HasAttachment = HasAttachment;
function From(str) {
    return { value: "from:(" + str + ")" };
}
exports.From = From;
function Subject(str) {
    return { value: "subject:(" + str + ")" };
}
exports.Subject = Subject;
function Label(str) {
    return { value: "label:(" + str + ")" };
}
exports.Label = Label;
function AND() {
    var nodes = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        nodes[_i] = arguments[_i];
    }
    return {
        $and: nodes,
    };
}
exports.AND = AND;
function NOT(query) {
    return ({ $not: query });
}
exports.NOT = NOT;
function OR() {
    var nodes = [];
    for (var _i = 0; _i < arguments.length; _i++) {
        nodes[_i] = arguments[_i];
    }
    return {
        $or: nodes,
    };
}
exports.OR = OR;
function isAndNode(node) {
    return '$and' in node;
}
exports.isAndNode = isAndNode;
function isOrNode(node) {
    return '$or' in node;
}
exports.isOrNode = isOrNode;
function isNotNode(node) {
    return '$not' in node;
}
exports.isNotNode = isNotNode;
function isLeafNode(node) {
    return !isAndNode(node) && !isOrNode(node);
}
exports.isLeafNode = isLeafNode;
function parse(query) {
    var parser = new nearley_1.Parser(nearley_1.Grammar.fromCompiled(grammar), { keepHistory: true });
    parser.feed(query);
    var results = parser.finish();
    return results[0];
}
exports.parse = parse;
function serialise(node) {
    if (isLeafNode(node)) {
        return node.value;
    }
    else if (isOrNode(node)) {
        if (node.$or.length < 2) {
            return serialise(node.$or[0]);
        }
        return '(' + node.$or.map(serialise).join(' OR ') + ')';
    }
    else if (isAndNode(node)) {
        if (node.$and.length < 2) {
            return serialise(node.$and[0]);
        }
        return '(' + node.$and.map(serialise).join(' AND ') + ')';
    }
    else {
        return 'NOT ' + serialise(node.$not);
    }
}
exports.serialise = serialise;
